require 'spec_helper'
require_relative '../lib/kinetic/dsl'
require_relative '../lib/kinetic/errors'
class TestClass
  extend Kinetic::DSL
end

describe Kinetic::DSL do
  # setting this to open brackes helps us test the returns true case

  let(:config) { { key: 'value' } }

  before :each do
    stub_const("Kinetic::Configuration", config)
  end

  subject do
    TestClass
  end

  before :each do
    subject.stub(config: config)
  end

  describe '::set' do

    it 'returns true' do
      subject.set('foo', 'baz').should == true
    end

    it 'sets the configuration value' do
      subject.set(:baz, 'bar')
      subject.send(:config)[:baz].should == 'bar'
    end

    it 'symbolizes strings into keys' do
      subject.set('foo', 'bar')
      subject.send(:config)[:foo].should == 'bar'
    end

  end

  describe '::get' do

    it 'returns the value of the config key' do
      config[:foo] = 'bar'
      subject.get(:foo).should == 'bar'
    end

    it 'symbolizes the key' do
      config[:baz] = 'bar'
      subject.get('baz').should == 'bar'
    end

  end

  describe '::get!' do

    it 'raises Kinetic::Errors::MissingConfigurationValue if the value is not present' do
      expect do
        subject.send(:get!, :nonexistentkey)
      end.to raise_error(Kinetic::Errors::MissingConfigurationValue)
    end

    it 'returns the value if the value is present and does not raise an exception' do
      config[:existentkey] = 'foo'
      expect do
        subject.get!(:existentkey).should == 'foo'
      end.not_to raise_error
    end

    it 'symbolizes the key' do
      config[:akey] = 'foo'
      expect do
        subject.get!(:akey)
      end.not_to raise_error
    end

  end

  describe '::logger' do

    let(:logger) { double('Logger') }

    after :each do
      subject.instance_variable_set(:@logger, nil)
    end

    it 'returns calls reopen_logger and returns the result if @logger is nil' do
      subject.instance_variable_set(:@logger, nil)
      subject.should_receive(:reopen_logger).and_return(logger)
      subject.logger.should == logger
    end

    it 'returns the existing logger if already set' do
      subject.instance_variable_set(:@logger, logger)
      subject.should_not_receive(:reopen_logger)
      subject.logger.should == logger
    end

  end

end
